#include <Wire.h>
#include <Adafruit_RGBLCDShield.h>
#include <Adafruit_PWMServoDriver.h>
#include <utility/Adafruit_MCP23017.h>
#include <Servo.h>
#include <Adafruit_MotorShield.h>
#include "utility/Adafruit_MS_PWMServoDriver.h"
#include "RTClib.h"
#include <EEPROM.h>

RTC_DS3231 rtc;

Adafruit_MotorShield AFMS = Adafruit_MotorShield(); 
Adafruit_StepperMotor *track = AFMS.getStepper(200, 2);// Connect a stepper motor with 200 steps per revolution (1.8 degree) to motor port #2 (M3 and M4)
Adafruit_StepperMotor *gantry = AFMS.getStepper(200, 1);

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
#define WHITE 0x7

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
uint8_t can = 0;
uint8_t flip = 2;
uint8_t grab = 4;

long timeout = 0;
int debounce = 250;
int trackend = 34;
int gantryend = 36;
int pump = 26;
int opener = 28;
int dir = 24;
int cansense = 32;
int red = 23;
int green = 25;

int set1 = 0;
int set2 = 0;
int set3 = 0;
int set4 = 0;
int hours = 0;
int minutes = 0;
int tripped = 0;
long tripreset = 0;

int addr = 0;
int i = 0;
int selection = 0;
int digit[5] = {0,0,0,0,0};

void setup() 
{  
  if (! rtc.begin()) 
  {
    lcd.print("Couldn't find RTC");
    while (1);
  }

  if (rtc.lostPower()) 
  {
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));   
  }
  
  pinMode(trackend,INPUT);
  pinMode(gantryend,INPUT);
  pinMode(cansense,INPUT);
  pinMode(dir, OUTPUT);
  pinMode(pump, OUTPUT);
  pinMode(opener, OUTPUT);
  pinMode(red,OUTPUT);
  pinMode(green,OUTPUT);
  
  lcd.begin(16, 2);  
  lcd.setBacklight(WHITE);
  digitalWrite(red, HIGH);
  digitalWrite(green, HIGH);

  pwm.begin();  
  pwm.setPWMFreq(60);
  
  AFMS.begin();  // create with the default frequency 1.6KHz  
  TWBR = ((F_CPU /400000l) - 16) / 2; // Change the i2c clock to 400KHz
  track->setSpeed(300);  
  
  pwm.setPWM(can, 0, 550);
  pwm.setPWM(flip, 0, 548);
  pwm.setPWM(grab, 0, 550);

  set1 = EEPROM.read(0); 
  set2 = EEPROM.read(1);
  set3 = EEPROM.read(2);
  set4 = EEPROM.read(3);

  hours = (set1*10) + set2;
  minutes = (set3*10) + set4;
    
  lcd.clear();
  lcd.setCursor(3,0);
  lcd.print("KetMeshin");
  lcd.setCursor(0,1);
  lcd.print("Vikram Hao, Inc.");
 
  Home_Gantry(); 
  Home_Track();
  track->release();
  gantry->release();

  pwm.setPWM(flip, 0, 0);
  pwm.setPWM(grab, 0, 0);
  
  lcd.clear();   
}

void loop() 
{

  UX();
 
}

void UX()
{
  if (digitalRead(cansense)==LOW)
  {
    can_error();
  } 
  DateTime now = rtc.now();
   
  lcd.setCursor(0,0);  
  
  lcd.print(now.month(),DEC); 
  lcd.print("/");
  lcd.print(now.day(),DEC);
  lcd.print("/"); 
  lcd.print(now.year(),DEC);
  lcd.setCursor(11,0);
  if (now.hour()<10)
  {
    lcd.print("0");
  }
  lcd.print(now.hour(), DEC);
  lcd.print(':');
  
  if (now.minute()<10)
  {
    lcd.print("0");
  }
  lcd.print(now.minute(), DEC); 
  lcd.setCursor(0,1);
  lcd.print("Feed Time:");
  lcd.print(" ");
  lcd.print(set1);
  lcd.print(set2);
  lcd.print(':');
  lcd.print(set3);
  lcd.print(set4);  
  
  if (hours==now.hour())
  {
    if (minutes==now.minute())
    {
      if (tripped==0)
      {
        tripped = 1;
        Dispense_Can();        
        tripreset = millis();        
      }
    }
  }
  
  if (millis()-tripreset>62000)
  {
    tripped = 0;
  }
  
  delay(100);
  
  digitalWrite(green, LOW);
  
  uint8_t buttons = lcd.readButtons();

  if (buttons) 
  {
    if (buttons & BUTTON_UP) {
           
    }
    if (buttons & BUTTON_DOWN) {
          
    }
    if (buttons & BUTTON_LEFT) {
            
    }
    if (buttons & BUTTON_RIGHT) {
            
    }
    if (buttons & BUTTON_SELECT) {
      delay(debounce); 
      Main_Menu();
      return;                       
    }
  }
}

void Main_Menu()
{
      selection=1;
      lcd.clear();
      lcd.setCursor(1,0);
      lcd.print("Set Time");
      lcd.setCursor(1,1);
      lcd.print("Operations");
      lcd.setCursor(0,0);
      lcd.print(">");
      
      delay(debounce);      
   while(1)
   {  
     uint8_t buttons = lcd.readButtons();

  if (buttons) 
  {
    if (buttons & BUTTON_UP) 
    {
          if (selection==2)
          {
          lcd.setCursor(0,1);
          lcd.print(" ");
          lcd.setCursor(0,0);
          lcd.print(">"); 
          selection=1;
          }
    }
    if (buttons & BUTTON_DOWN) {
          selection=2;
          lcd.setCursor(0,0);
          lcd.print(" ");
          lcd.setCursor(0,1);
          lcd.print(">");
    }
    if (buttons & BUTTON_LEFT) {
            return;
    }
    if (buttons & BUTTON_RIGHT) {
            
    }
    if (buttons & BUTTON_SELECT) {
        if (selection==1)
        {
           delay(debounce);
           Set_Time();
           return;                            
        }
         if (selection==2)
        {
           delay(debounce);
           extra_menu(); 
           lcd.clear();
           return;                        
        }
    }
  }
   }        
}

void Set_Time()
{
      i=0;
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print(set1);
      lcd.print(set2);
      lcd.print(':');
      lcd.print(set3);
      lcd.print(set4);
      lcd.setCursor(0,1);
      lcd.print("^");

      digit[0]=set1;
      digit[1]=set2;
      digit[3]=set3;
      digit[4]=set4;
                  
   while(1)
   {  
     uint8_t buttons = lcd.readButtons();

  if (buttons) 
  {
    if (buttons & BUTTON_UP) 
    {
          lcd.setCursor(i,0);          
          if (digit[i]<9)
          {
            digit[i]++;
          }
          lcd.print(digit[i]);        
          delay(debounce);
    }
    if (buttons & BUTTON_DOWN) {
          lcd.setCursor(i,0);          
          if (digit[i]>0)
          {
            digit[i]--;
          }
          lcd.print(digit[i]); 
          delay(debounce);
    }
    if (buttons & BUTTON_LEFT) {
            if (i>0)
            {
            lcd.setCursor(i,1);
            lcd.print(" ");   
            i--;
            lcd.setCursor(i,1);
            lcd.print("^");
            }
            
            delay(debounce);
    }
    if (buttons & BUTTON_RIGHT) {
     if (i<4)
     {
      lcd.setCursor(i,1);
      lcd.print(" "); 
      i++; 
      lcd.setCursor(i,1);
      lcd.print("^");
      }
     
     delay(debounce);      
    }
    if (buttons & BUTTON_SELECT) {

    EEPROM.write(0, digit[0]);
    EEPROM.write(1, digit[1]);
    EEPROM.write(2, digit[3]); 
    EEPROM.write(3, digit[4]);
    set1 = EEPROM.read(0); 
    set2 = EEPROM.read(1);
    set3 = EEPROM.read(2);
    set4 = EEPROM.read(3); 
    
    hours = (set1*10) + set2;
    minutes = (set3*10) + set4;   
    
    delay(debounce);
    return;
    
    }
  }
   }        
}

void extra_menu()
{
      selection=1;
      lcd.clear();
      lcd.setCursor(1,0);
      lcd.print("Dispense Can");
      lcd.setCursor(1,1);
      lcd.print("Empty Hopper");
      lcd.setCursor(0,0);
      lcd.print(">");
      
      delay(debounce);      
   while(1)
   {  
     uint8_t buttons = lcd.readButtons();

  if (buttons) 
  {
    if (buttons & BUTTON_UP) 
    {
          if (selection==2)
          {
          lcd.setCursor(0,1);
          lcd.print(" ");
          lcd.setCursor(0,0);
          lcd.print(">"); 
          selection=1;
          }
    }
    if (buttons & BUTTON_DOWN) {
          selection=2;
          lcd.setCursor(0,0);
          lcd.print(" ");
          lcd.setCursor(0,1);
          lcd.print(">");
    }
    if (buttons & BUTTON_LEFT) {
            return;
    }
    if (buttons & BUTTON_RIGHT) {
            
    }
    if (buttons & BUTTON_SELECT) {
        if (selection==1)
        {
           delay(debounce);
           Dispense_Can();
           lcd.clear();
           return;                            
        }
         if (selection==2)
        {
           delay(debounce);
           clear_cans(); 
           lcd.clear();
           return;                        
        }
    }
  }
   }  
}

void clear_cans()
{
  Home_Gantry();
  Home_Track();
  pwm.setPWM(flip, 0, 548);
  pwm.setPWM(grab, 0, 550);
  lcd.clear();
  digitalWrite(green, HIGH);
  digitalWrite(red, LOW);
  lcd.setCursor(0,0); 
  lcd.print("Clearing Hopper"); 

   while(digitalRead(cansense)==HIGH)
   {
    pwm.setPWM(can, 0, 150);
    delay(1500);
    pwm.setPWM(can, 0, 550);
    pwm.setPWM(grab, 0, 245);
    track->step(1050, FORWARD, DOUBLE);
    delay(1000);
    pwm.setPWM(grab, 0, 550);
    delay(1000);
    track->step(1050, BACKWARD, DOUBLE);
    delay(500);
   }
   digitalWrite(red, HIGH);
   digitalWrite(green, LOW);
   pwm.setPWM(flip, 0, 0);
   pwm.setPWM(grab, 0, 0);
   track->release();
   gantry->release();
   return;   
}

void can_error()
{
  lcd.clear();
  digitalWrite(green, HIGH);
  lcd.setCursor(5,0); 
  lcd.print("Error:");
  lcd.setCursor(2,1);
  lcd.print("Refill Cans");  
  
  while(digitalRead(cansense)==LOW)
  {
    digitalWrite(red, LOW);
    delay(1000);
    digitalWrite(red, HIGH);
    delay(1000);
  }
  digitalWrite(red, HIGH);
  digitalWrite(green, LOW);
  lcd.clear();
  return;
}

void Dispense_Can()
{
  pwm.setPWM(flip, 0, 548);
  pwm.setPWM(grab, 0, 550);
  digitalWrite(green, HIGH);
  digitalWrite(red, LOW);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Dispensing...");
  Home_Track();
  Home_Gantry();
  pwm.setPWM(can, 0, 150);
  delay(1500);
  pwm.setPWM(can, 0, 550);
  pwm.setPWM(grab, 0, 245);
  track->step(575, FORWARD, DOUBLE);
  delay(1000);
  Open_Lid();
  track->step(470, FORWARD, DOUBLE);  
  delay(500); 
  Remove_Lid();
  track->step(305, FORWARD, DOUBLE);
  delay(500);
  Flip_Can();
  track->step(300, BACKWARD, DOUBLE);
  pwm.setPWM(grab, 0, 550);
  delay(2000);   
  Drop_Can();
  Home_Track();   
  delay(1000);
  digitalWrite(red, HIGH);
  digitalWrite(green, LOW);
  pwm.setPWM(flip, 0, 0);
  pwm.setPWM(grab, 0, 0); 
  track->release();
  gantry->release();
  lcd.clear();   
  return;
}

void Open_Lid()
{
  pwm.setPWM(grab, 0, 300);
  track->step(10, BACKWARD, DOUBLE);
  gantry->step(11900, FORWARD, DOUBLE);  
  delay(500);
  digitalWrite(dir,HIGH);
  digitalWrite(opener,HIGH);
  delay(30000); 
  digitalWrite(dir,LOW); 
  delay(1000); 
  digitalWrite(opener,LOW);
  track->step(10, FORWARD, DOUBLE);
  pwm.setPWM(grab, 0, 245);
  gantry->step(5000, BACKWARD, DOUBLE);  
}

void Remove_Lid()
{
  gantry->step(4500, FORWARD, DOUBLE);
  delay(500);
  digitalWrite(pump,HIGH);
  delay(1000);    
  gantry->step(4800, BACKWARD, DOUBLE);
  track->step(750, BACKWARD, DOUBLE);
  digitalWrite(pump,LOW);  
  delay(3000);
  Home_Gantry();
  track->step(750, FORWARD, DOUBLE);
}
void Home_Track()
{
  if(digitalRead(trackend)==HIGH)
  {    
    track->step(50, FORWARD, DOUBLE); 
    delay(100); 
    timeout=millis();        
    while (digitalRead(trackend)==HIGH)
    {
      track->step(10, BACKWARD, DOUBLE);
      if ((millis()-timeout)>6000)
      {
          track_error();
      }
    }
    track->step(50, FORWARD, DOUBLE);       
  }
}
void Home_Gantry()
{  
   if(digitalRead(gantryend)==HIGH)
  {    
    gantry->step(500, FORWARD, DOUBLE); 
    delay(100);
    timeout=millis(); 
    while (digitalRead(gantryend)==HIGH)
    {
      gantry->step(10, BACKWARD, DOUBLE);
      if ((millis()-timeout)>20000)
      {
        gantry_error();
      }
    }
    gantry->step(1000, FORWARD, DOUBLE);        
  }
  return;
}
void track_error()
{
  lcd.clear();
  digitalWrite(green, HIGH);
  lcd.setCursor(5,0); 
  lcd.print("Error:");
  lcd.setCursor(2,1);
  lcd.print("Track Jammed");
  
  while(1)
  {
    digitalWrite(red, LOW);
    delay(1000);
    digitalWrite(red, HIGH);
    delay(1000);
  }
}

void gantry_error()
{
  lcd.clear();
  digitalWrite(green, HIGH);
  lcd.setCursor(5,0); 
  lcd.print("Error:");
  lcd.setCursor(1,1);
  lcd.print("Gantry Jammed");
  
  while(1)
  {
    digitalWrite(red, LOW);
    delay(1000);
    digitalWrite(red, HIGH);
    delay(1000);
  }
}
  
void Flip_Can()
{
    int del = 150;
    
    pwm.setPWM(flip, 0, 125);
    delay(5000);  
    
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);    
    delay(20000);

    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);
    delay(del);
    pwm.setPWM(flip, 0, 205);
    delay(del);
    pwm.setPWM(flip, 0, 125);    
    delay(20000);
        
    pwm.setPWM(flip, 0, 548);  
    delay(1000);  
}

void Drop_Can()
{
  pwm.setPWM(flip, 0, 480); 
  delay(100);
  pwm.setPWM(flip, 0, 550); 
  delay(100);
  pwm.setPWM(flip, 0, 480); 
  delay(100);
  pwm.setPWM(flip, 0, 550);
  delay(1000);
  pwm.setPWM(flip, 0, 480); 
  delay(100);
  pwm.setPWM(flip, 0, 550); 
  delay(100);
  pwm.setPWM(flip, 0, 480); 
  delay(100);
  pwm.setPWM(flip, 0, 548);
  delay(2000);
}


